import java.io.*;
import java.util.*;

public class TestSOM{
	
	public static Gene[] read_dataset(String fileName){
		BufferedReader reader;
		int numGenes;
		String[] splitLine;
		double[] exprValues;
		Gene[] genes = null;
		
		try{
			reader = new BufferedReader(new FileReader(fileName));
			
			// Count the number of lines to determine how many genes are present
			for(numGenes = 0; reader.readLine( ) != null; numGenes++);
			reader.close();
			
			reader = new BufferedReader(new FileReader(fileName));
			
			// Allocate space for the genes array
			genes = new Gene[numGenes];
			exprValues = null;
			
			// Now, read in each line and create the corresponding Gene object
			for(int i = 0; i < genes.length; i++){
				String line = reader.readLine();
				// The files are tab-delimited, so split on tabs (\t)
				splitLine = line.split( "\t" );
				
				// The first entry in the parts array is the gene name, the rest
				// are expression levels for that gene
				if((exprValues == null) || (exprValues.length != 
											(splitLine.length - 1)))
					exprValues = new double[splitLine.length - 1];
				
				for(int j = 0; j < exprValues.length; j++)
					exprValues[j] = Double.parseDouble(splitLine[j + 1]);
				
				// Finally, create the Gene in the array
				genes[i] = new Gene(splitLine[0], exprValues); 
			}
			
			// Lastly, close the file
			reader.close(); 
		}catch(FileNotFoundException e){
			System.out.println( "ERROR:  Unable to locate " + fileName + "." );
			System.exit( 0 ); 
		}catch(Exception e){
			System.out.println( "ERROR:  Unable to read from " + fileName + "." );
			System.exit( 0 );
		} 
		
		return genes;
	}
	
	public static void main(String[] args){
		Gene[] genes = TestSOM.read_dataset(args[0]);
		int M = Integer.parseInt(args[1]);
		int N = Integer.parseInt(args[2]);
		int NUM_ITERATIONS = Integer.parseInt(args[3]);
		double INITIAL_LEARNING_RATE = Double.parseDouble(args[4]);
		
		//initializations for drawing
		double MNRatio = (double)M / N;
		int X = (int) (400*MNRatio);
		int Y = 400;
		
		//you may turn off these lines to disable drawing
		StdDraw.setCanvasSize(X, Y);
		StdDraw.setXscale(0, X);
		StdDraw.setYscale(0, Y);
		
		//starts testing
		SOM som = new SOM(genes, M, N, NUM_ITERATIONS, INITIAL_LEARNING_RATE);
		som.do_cluster();
		System.out.println(som);

		//saves the final image in addition to displaying animations.
		//(optional)
		//new Visualizer(som, args[0] + "." + args[1] + "." + args[2] + "." + 
		//	args[3] + "." + args[4]);
		
	}
}
