import java.io.*;
import java.awt.Color;
import java.util.*;

public class Visualizer{

	SOM s;
	
	double[] max_val; //for scaling up the values to 0-255
	double[] min_val; //for scaling up the values to 0-255
	
	private void init(){
		SOMNode[] sn = s.get_map();
		max_val = new double[sn[0].get_weight_vector().length];
		min_val = new double[sn[0].get_weight_vector().length];
		
		Vector<Vector<Double>> val = new Vector<Vector<Double>>();
		for(int i=0; i<sn[0].get_weight_vector().length; i++){
			val.add(new Vector<Double>());
		}
		
		for(int i=0; i<sn.length; i++){
			for(int j=0; j<max_val.length; j++){
				val.get(j).add(sn[i].get_weight_vector()[j]);
			}
		}
		
		for(int j=0; j<val.size(); j++){
			Collections.sort(val.get(j));
			max_val[j] = val.get(j).get((int)(val.get(j).size()*0.80));
			min_val[j] = val.get(j).get((int)(val.get(j).size()*0.20));
		}
		
	}
	
	//displays a drawing of the map
	public Visualizer(SOM s){
		this.s = s;
		init();
		
		SOMNode[] sn = s.get_map();

		//supposes that canvas size has been set already
		int X = (int) StdDraw.getCanvasWidth();
		int Y = (int) StdDraw.getCanvasHeight();
		
		double unit_X = X / s.get_M();
		double unit_Y = Y / s.get_N();

		
		for(int i=0; i<sn.length; i++){
			double[] weight_vector = sn[i].get_weight_vector();
			int[] component = new int[sn[i].get_weight_vector().length];
			
			for(int j=0; j<component.length; j++){
				double v = -1.0*min_val[j];
				double true_max = v+max_val[j];
				component[j] = (int)(1.0*(weight_vector[j]+v)/true_max*255);
				if(component[j]<0.0){ component[j] = 0;}
				if(component[j]>255.0){ component[j] = 255;}
				
			}
			
			Color c = new Color(component[0], component[1], component[2]);
			
			int x = sn[i].get_x();
			int y = sn[i].get_y();
			
			StdDraw.setPenColor(c);
			StdDraw.filledRectangle((x+0.5)*unit_X, Y-(y+0.5)*unit_Y, unit_X/2+0.5, unit_Y/2+0.5);
		}
		
	}
	

	//saves the drawing to an image file
	public Visualizer(SOM s, String f) {
		this.s = s;
		int u = 10;
		init();
			
		SOMNode[] sn = s.get_map();
				
		Picture p = new Picture(u*s.get_M(), u*s.get_N());
					
		for(int i=0; i<sn.length; i++){
			double[] weight_vector = sn[i].get_weight_vector();
			int[] component = new int[sn[i].get_weight_vector().length];
				
			for(int j=0; j<component.length; j++){
				double v = -1.0*min_val[j];
				double true_max = v+max_val[j];
				component[j] = (int)(1.0*(weight_vector[j]+v)/true_max*255);
				if(component[j]<0.0){ component[j] = 0;}
				if(component[j]>255.0){ component[j] = 255;}
				
			}
				
			Color c = new Color(component[0], component[1], component[2]);

			int x = sn[i].get_x();
			int y = sn[i].get_y();
			p.set(x*u, y*u, c);
			for(int k=0; k<u; k++){
				for(int kk=0; kk<u; kk++){
					p.set(x*u+k, y*u+kk, c);
				}
			}
			
		}
			
		p.save(f+".png");
			
		
	}

}
		
